/*
 * This Software (C) Copyright David Goodwin, 2008, 2009.
 * 
 *   This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package iasel;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import javax.swing.JOptionPane;
import org.ini4j.Ini;
import org.ini4j.Ini.Section;

/** Holds some global variables, etc.
 *
 * @author David
 */
public class globals {   
    /** Application version information
     */
    public static final String appName = "IASEL";
    public static final Integer appMajorVersion = 1;
    public static final Integer appMinorVersion = 1;
    public static final Integer appPatchLevel = 0;
    
    /** The location of the application configuration file.
     */
    public static final String settingsFileName = "settings.ini";
    
    /** The location of the area calculation profile file.
     */
    public static final String areaFileName = "area_profiles.ini";
    
    /** This is used to access the area profile information file
     */
    public static final AreaFile areaProfiles = new AreaFile();
    
    /** When TRUE, area calculations will be enabled.
     */
    public static boolean AreaCalcEnable = true;
    
    /** The settings file
     */
    private static Ini sIfile = null;
    
    public static boolean settings_loaded = false;
    
    /** Not allowed to instantitate this */
    protected globals() {
    }
    
    /** Gets a key from the settings file.
     * @param key The key to get the value for.
     * @param sDefaultValue The value to return if either the key, or the value does not exist.
     * @return The value of the specified key.
     */
    private static String getValue(final String key, final String sDefaultValue) {
        String tmp;
        String sectName;
        String keyName;
        if (sIfile == null) {
            return sDefaultValue;
        }
        Section sect = sIfile.new Section("");
        
        /* Figure out the section and key names from the input value */
        final String[] parts = key.split("[\\\\]");
        if (parts.length > 1) {
            sectName = parts[0];
            keyName = parts[1];
        } else if (parts.length == 1) {
            sectName = "general";
            keyName = parts[0];
        } else {
            return sDefaultValue;
        }
         
        /* Make sure our input parameters are allowed */
        if(key == null || sDefaultValue == null) {
            return null;
        }
        
        sect = sIfile.get(sectName);
        
        /* If the section doesn't exist, return the default value. */
        if(sect == null) {
            return sDefaultValue;
        }
        
        /* Now try and get the key value*/
        tmp = sect.get(keyName);

        /* If the key doesn't exist, return the default value*/
        if(tmp == null) {
            tmp = sDefaultValue;
        }
        
        /* Return the result. */
        return tmp;
    }            
    
    /** Sets a value in the settings file.
     * @param key The key to set the value for.
     * @param value The value to set the key to
     */
    private static void setValue(final String key, final String value) {
        String tmp;
        String sectName;
        String keyName;
        if (sIfile == null) {
            return;
        }
        Section sect = sIfile.new Section("");
        
        /* Figure out the section and key names from the input value */
        final String[] parts = key.split("[\\\\]");
        if (parts.length > 1) {
            sectName = parts[0];
            keyName = parts[1];
        } else if (parts.length == 1) {
            sectName = "general";
            keyName = parts[0];
        } else {
            return;
        }
         
        /* Make sure our input parameters are allowed */
        if(key == null || value == null) {
            return;
        }
        
        sect = sIfile.get(sectName);
        
        /* If the section doesn't exist, we must create it. */
        if(sect == null) {
            sect = sIfile.add(sectName);
        }
        
        /* Now try and set the key value*/
        sect.put(keyName, value);
        
        /* And then save the settings file */
        save_settings();
    }                
    
    
    /** Returns the application name string. Something like:
     * "IASEL v1.0-2". The patch level is excluded if its value
     * is set to 0.
     * 
     * @return The application name string.
     */
    public static String getAppNameString() {
        String appString = globals.appName + " v" + 
                globals.appMajorVersion.toString() +
                "." + globals.appMinorVersion.toString();
        if (globals.appPatchLevel != 0) 
            appString += "-" + globals.appPatchLevel.toString();
        return appString;
    }
    
    /** Loads the settings file
     */
    public static boolean load_settings() {
        try {
            sIfile = new Ini(new FileReader(new File(globals.settingsFileName)));
            settings_loaded = true;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Unable to open Settings file" +
                    ". Default settings will be used.", "Could not open file",
                    JOptionPane.ERROR_MESSAGE);
            settings_loaded = false;
        }
        return settings_loaded;
    }
    
    public static boolean save_settings() {
        try {
            FileWriter fw = new FileWriter(new File(globals.settingsFileName));
            sIfile.store(fw);
            return true;
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null, "Unable to update Settings" +
                    "file. Settings will not be updated", "Could not write to file",
                    JOptionPane.ERROR_MESSAGE);
            return false;
        }  
    }
    
    /** Gets the default image open directory from the settings file and
     * returns it.
     * 
     * @return Default image open dialog directory
     */
    public static String get_img_open_dir() {
        
        // If the settings arent loaded already
        if (!settings_loaded) 
            // Then load them
            if (!load_settings())
                // If that fails then return
                return "";
        
        return getValue("locations\\img-open","");
    }
    
    /** Sets a new default image open directory if updates are enabled in the
     * settings file
     * 
     * @param newdir The new default location
     */
    public static void set_img_open_dir(String newdir) {
        String update_allowed = getValue("locations\\update","false").toUpperCase();
        if (!update_allowed.startsWith("TRUE"))
            return;
        
        setValue("locations\\img-open", newdir);
    }
    
    /** Gets the default export directory and returns it.
     * 
     * @return The default export directory.
     */
    public static String get_export_open_dir() {
        // If the settings arent loaded already
        if (!settings_loaded) 
            // Then load them
            if (!load_settings())
                // If that fails then return
                return "";
        
        return getValue("locations\\export","");
    }
    
    /** Attempts to set the default export directory if updates are enabled in
     * the configuration file.
     * 
     * @param newdir The new default export directory.
     */
    public static void set_export_open_dir(String newdir) {
        String update_allowed = getValue("locations\\update","false").toUpperCase();
        if (!update_allowed.startsWith("TRUE"))
            return;
        
        setValue("locations\\export", newdir);
    }
}
